"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bus = void 0;
var core_1 = require("@we-scrum/core");
var Bus = /** @class */ (function () {
    function Bus() {
        this.projections = [];
        this.commandHandlers = new Map();
        this.eventHandlers = new Map();
    }
    Bus.prototype.send = function (command, context) {
        return __awaiter(this, void 0, void 0, function () {
            var handler;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        core_1.Logger.info("Handling command", command);
                        handler = this.commandHandlers.get(command.type);
                        if (!handler) {
                            throw new Error("No command handler for " + command.type);
                        }
                        core_1.Logger.info("Run command handler " + handler.constructor.name);
                        return [4 /*yield*/, handler.handle(command, context)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    Bus.prototype.publish = function (event, mode, context) {
        return __awaiter(this, void 0, void 0, function () {
            var key, handlers, _i, handlers_1, handler, projections, _a, projections_1, projection;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        core_1.Logger.info("Handling event", event);
                        key = this.getDictionaryKey(event.type, mode);
                        handlers = this.eventHandlers.get(key) || [];
                        _i = 0, handlers_1 = handlers;
                        _b.label = 1;
                    case 1:
                        if (!(_i < handlers_1.length)) return [3 /*break*/, 4];
                        handler = handlers_1[_i];
                        core_1.Logger.info("Run event handler " + handler.constructor.name);
                        return [4 /*yield*/, handler.handle(event, context)];
                    case 2:
                        _b.sent();
                        _b.label = 3;
                    case 3:
                        _i++;
                        return [3 /*break*/, 1];
                    case 4:
                        projections = this.projections.filter(function (p) { return p.isRegisteredEvent(event.type, mode); });
                        _a = 0, projections_1 = projections;
                        _b.label = 5;
                    case 5:
                        if (!(_a < projections_1.length)) return [3 /*break*/, 8];
                        projection = projections_1[_a];
                        core_1.Logger.info("Run projection " + projection.constructor.name);
                        return [4 /*yield*/, projection.handle(event, mode, context)];
                    case 6:
                        _b.sent();
                        _b.label = 7;
                    case 7:
                        _a++;
                        return [3 /*break*/, 5];
                    case 8: return [2 /*return*/];
                }
            });
        });
    };
    Bus.prototype.onCommand = function (commandType, handler) {
        var commandName = (new commandType()).type;
        this.commandHandlers.set(commandName, handler);
    };
    Bus.prototype.onEvent = function (eventType, handler, mode) {
        var key = this.getDictionaryKey((new eventType()).type, mode);
        if (!this.eventHandlers.has(key)) {
            this.eventHandlers.set(key, []);
        }
        this.eventHandlers.get(key).push(handler);
    };
    Bus.prototype.onProjection = function (projection) {
        this.projections.push(projection);
    };
    Bus.prototype.getDictionaryKey = function (eventType, mode) {
        return eventType + "_" + mode.toString();
    };
    return Bus;
}());
exports.Bus = Bus;
//# sourceMappingURL=bus.js.map